import { ListIndexesOptions } from 'mongodb';
import { FindOneAndUpsertCommand, MongoAggregateCommand, MongoCountDocumentsCommand, MongoCreateIndexesCommand, MongoDropIndexCommand, MongoFindCommand, MongoFindOneAndUpdateCommand, MongoFindOneCommand, MongoInsertManyCommand, MongoInsertOneCommand, MongoUpdateOneCommand, MongoUpsertCommand } from '../types';
import { MongoDocument, MongoDatabase } from './mongo';
import { CollectionOptions, EntityMetadata } from './types';
export declare class Collection<T extends MongoDocument> {
    private readonly collection;
    private readonly metadata;
    private readonly options;
    constructor(database: MongoDatabase, metadata: EntityMetadata, options?: CollectionOptions);
    find(command?: MongoFindCommand<T>): import("mongodb").FindCursor<import("mongodb").WithId<T>>;
    findOne(command?: MongoFindOneCommand<T>): Promise<import("mongodb").WithId<T> | null>;
    findOneAndUpdate(command?: MongoFindOneAndUpdateCommand<T>): Promise<import("mongodb").WithId<T> | null>;
    updateOne(command?: MongoUpdateOneCommand<T>): Promise<import("mongodb").UpdateResult<T>>;
    insertOne(command: MongoInsertOneCommand<T>): Promise<{
        _id: import("mongodb").InferIdType<T>;
    } & import("mongodb").OptionalUnlessRequiredId<T>>;
    insertMany(command: MongoInsertManyCommand<T>): Promise<({
        _id: import("mongodb").InferIdType<T>;
    } & import("mongodb").OptionalUnlessRequiredId<T>)[]>;
    upsert(command?: MongoUpsertCommand<T>): Promise<import("mongodb").UpdateResult<T>>;
    findOneAndUpsert(command?: FindOneAndUpsertCommand<T>): Promise<import("mongodb").WithId<T> | null>;
    aggregate(command?: MongoAggregateCommand<T>): import("mongodb").AggregationCursor<MongoDocument>;
    countDocuments(command?: MongoCountDocumentsCommand<T>): Promise<number>;
    indexes(options?: ListIndexesOptions): Promise<import("mongodb").IndexDescriptionInfo[]>;
    dropIndex(command: MongoDropIndexCommand): Promise<MongoDocument>;
    createIndexes(command: MongoCreateIndexesCommand): Promise<string[]>;
}
