"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
class Collection {
    collection;
    metadata;
    options;
    constructor(database, metadata, options) {
        this.collection = database.collection(metadata.collectionName, options);
        this.metadata = metadata;
        this.options = options ?? {};
    }
    find(command) {
        return this.collection.find(command?.where ?? {}, {
            session: this.options.session,
            ...command?.options
        });
    }
    findOne(command) {
        return this.collection.findOne(command?.where ?? {}, {
            session: this.options.session,
            ...command?.options
        });
    }
    findOneAndUpdate(command) {
        return this.collection.findOneAndUpdate(command?.where ?? {}, command?.document ?? {}, {
            session: this.options.session,
            ...command?.options
        });
    }
    updateOne(command) {
        return this.collection.updateOne(command?.where ?? {}, command?.document ?? {}, command?.options);
    }
    insertOne(command) {
        return this.collection
            .insertOne(command.document, {
            session: this.options.session,
            ...command.options
        })
            .then(result => {
            if (!result.acknowledged) {
            }
            return {
                _id: result.insertedId,
                ...command.document
            };
        });
    }
    insertMany(command) {
        return this.collection
            .insertMany(command.documents, {
            session: this.options.session,
            ...command.options
        })
            .then(result => {
            if (!result.acknowledged) {
            }
            return command.documents.map((insertedDocument, documentIndex) => ({
                _id: result.insertedIds[documentIndex],
                ...insertedDocument
            }));
        });
    }
    upsert(command) {
        return this.collection.updateOne(command?.where ?? {}, command?.document ?? {}, {
            session: this.options.session,
            ...command?.options,
            upsert: true
        });
    }
    findOneAndUpsert(command) {
        return this.collection.findOneAndUpdate(command?.where ?? {}, command?.document ?? {}, {
            returnDocument: 'after',
            session: this.options.session,
            ...command?.options,
            upsert: true
        });
    }
    aggregate(command) {
        return this.collection.aggregate(command?.pipeline, {
            session: this.options.session,
            ...command?.options
        });
    }
    countDocuments(command) {
        return this.collection.countDocuments(command?.where, {
            session: this.options.session,
            ...command?.options
        });
    }
    indexes(options) {
        return this.collection.indexes(options);
    }
    dropIndex(command) {
        return this.collection.dropIndex(command.indexName, {
            ...command.options,
            session: this.options.session
        });
    }
    createIndexes(command) {
        return this.collection.createIndexes(command.specification, {
            ...command.options,
            session: this.options.session
        });
    }
}
exports.Collection = Collection;
//# sourceMappingURL=collection.js.map