"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Metadata = void 0;
const ramda_1 = require("ramda");
const utils_1 = require("./utils");
class Metadata {
    static define(target, metadataKey, data) {
        Reflect.defineMetadata(metadataKey, data, target);
    }
    static mergeDeep(target, metadataKey, data) {
        const currentMetadataValue = Reflect.getMetadata(metadataKey, target);
        const metadataValue = (0, ramda_1.mergeDeepWith)(utils_1.mergeObjectAndArray, currentMetadataValue, data);
        Reflect.defineMetadata(metadataKey, metadataValue, target);
        return metadataValue;
    }
    static get(target, metadataKey) {
        return Reflect.getMetadata(metadataKey, target);
    }
    static find(target, metadataKey) {
        return Reflect.getMetadata(metadataKey, target) ?? null;
    }
    static has(target, metadataKey) {
        return Reflect.hasMetadata(metadataKey, target);
    }
}
exports.Metadata = Metadata;
//# sourceMappingURL=metadata.js.map