import { InjectionToken } from '@nestjs/common';
import { ClientSession, CreateIndexesOptions } from 'mongodb';
import { MongoClientOptions, MongoCollectionOptions, MongoDatabaseOptions } from './mongo';
import { BSONType } from './enums';
export type Class<T = any> = new (...args: Array<any>) => T;
export type MixedList<T> = Array<T> | {
    [key: string]: T;
};
export type DataSourceSingleHost = {
    host: string;
    port: number;
};
export type DataSourceOptions = {
    uri?: string;
    hosts?: Array<DataSourceSingleHost>;
    database?: string;
    username?: string;
    password?: string;
    synchronize?: boolean;
    runMigrations?: boolean;
    entities?: MixedList<Class>;
    migrations?: MixedList<Class>;
    dbOptions?: MongoDatabaseOptions;
    clientOptions?: MongoClientOptions;
};
export type ForRootOptions = {
    global?: boolean;
    connectionName?: string;
    options?: DataSourceOptions;
};
export type ForRootAsyncOptions = {
    global?: boolean;
    connectionName?: string;
    inject?: Array<InjectionToken>;
    useFactory: (...args: Array<any>) => Promise<DataSourceOptions> | DataSourceOptions;
};
export type EntityColumnTypes = StringConstructor | NumberConstructor | BooleanConstructor | ObjectConstructor | Class | Record<string, any>;
export type EntityColumn = {
    type: EntityColumnTypes;
    parent: Class;
    propertyKey: string;
    options: EntityColumnOptions;
};
export type EntityColumnOptions = {
    type?: EntityColumnTypes;
    description?: string;
    array?: boolean;
    unique?: boolean;
    enum?: object;
    required?: boolean;
    index?: string | Array<string> | CreateIndexesOptions;
    nestedValueTypes?: Array<EntityColumnTypes>;
};
export type EntityOptions = {
    index?: Record<string, CreateIndexesOptions>;
    validationLevel?: 'off' | 'strict' | 'moderate';
    validationAction?: 'error' | 'warn';
    options?: CollectionOptions;
};
export type EntityMetadata = {
    entity: Class;
    collectionName: string | null;
    entityOptions: EntityOptions;
    entityColumns: Array<EntityColumn>;
};
export type EntityIndex = {
    keys: Array<string>;
    options: CreateIndexesOptions;
};
export type EntityColumnValidator = {
    type?: BSONType;
    bsonType?: BSONType;
    description?: string;
    minItems?: number;
    maxItems?: number;
    required?: Array<string>;
    enum?: Array<string>;
    additionalProperties?: boolean | EntityColumnValidator;
    items?: EntityColumnValidator;
    uniqueItems?: boolean;
    pattern?: string;
    properties?: Record<string, EntityColumnValidator>;
    patternProperties?: Record<string, string>;
};
export type MongoJsonSchema = {
    $jsonSchema?: EntityColumnValidator;
};
export type CollectionOptions = MongoCollectionOptions & {
    session?: ClientSession;
};
