import { mergeDeepWith } from 'ramda'
import { Class } from './types'
import { mergeObjectAndArray } from './utils'

export class Metadata {
    static define<T>(target: object, metadataKey: string, data: T) {
        Reflect.defineMetadata(metadataKey, data, target)
    }

    static mergeDeep<T extends object>(target: object, metadataKey: string, data: Partial<T>): T {
        const currentMetadataValue = Reflect.getMetadata(metadataKey, target)
        const metadataValue = mergeDeepWith(mergeObjectAndArray, currentMetadataValue, data)

        Reflect.defineMetadata(metadataKey, metadataValue, target)

        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return metadataValue
    }

    static get<T>(target: object, metadataKey: string): T {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return Reflect.getMetadata(metadataKey, target)
    }

    static find<T>(target: object, metadataKey: string): T | null {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return Reflect.getMetadata(metadataKey, target) ?? null
    }

    static has(target: object, metadataKey: string): target is Class {
        return Reflect.hasMetadata(metadataKey, target)
    }
}
