import { MongoDatabase, MongoDocument } from './mongo'
import { Class, CollectionOptions } from './types'
import { getEntityMetadata } from './utils'
import { Collection } from './collection'

export class TransactionManager {
    constructor(
        private readonly db: MongoDatabase,
        private readonly options?: CollectionOptions
    ) {}

    getCollection<T extends MongoDocument>(entity: Class<T>): Collection<T> {
        const metadata = getEntityMetadata(entity)

        if (!metadata.collectionName) {
            throw new Error(
                `Only entities can have collections. Please make sure your [class ${entity.name}] is properly decorated with @Entity(collectionName: string).`
            )
        }

        return new Collection(this.db, metadata, this.options)
    }
}
