import { Class, ENTITY_METADATA, EntityColumnOptions, EntityMetadata, Metadata } from 'lib/common'

export const Column = (options?: EntityColumnOptions) => (target: object, propertyKey: string) => {
    const type = Reflect.getMetadata('design:type', target, propertyKey)

    if (type === Array && !options?.type && !options?.enum) {
        const message = [
            `Unknown type for '${propertyKey}' in [class ${target.constructor.name}].`,
            `Please provide explicit type at @Column({ type: ... }) or @Column({ enum: ... }).`
        ].join('\r\n')

        throw new Error(message)
    }

    Metadata.mergeDeep<EntityMetadata>(target.constructor, ENTITY_METADATA, {
        entityColumns: [
            {
                propertyKey,
                type: options?.type ?? options?.enum ?? type,
                parent: target.constructor as Class,
                options: {
                    ...options,
                    array: type === Array || options?.array,
                    required: options?.required ?? true
                }
            }
        ]
    })
}
