import {
    AggregateOptions,
    BulkWriteOptions,
    ClientSessionOptions,
    CountDocumentsOptions,
    CreateIndexesOptions,
    DropIndexesOptions,
    Filter,
    FindOneAndUpdateOptions,
    FindOptions,
    IndexDescription,
    InsertOneOptions,
    OptionalUnlessRequiredId,
    UpdateFilter,
    UpdateOptions
} from 'mongodb'
import { MongoDocument, TransactionManager } from 'lib/common'

export type MongoFindCommand<T extends MongoDocument> = {
    where?: Filter<T>
    options?: FindOptions<T>
}

export type MongoFindOneCommand<T extends MongoDocument> = {
    where?: Filter<T>
    options?: Omit<FindOptions<T>, 'timeoutMode'>
}

export type MongoUpdateOneCommand<T extends MongoDocument> = {
    where?: Filter<T>
    document?: UpdateFilter<T>
    options?: UpdateOptions
}

export type MongoFindOneAndUpdateCommand<T extends MongoDocument> = {
    where?: Filter<T>
    document?: UpdateFilter<T>
    options?: FindOneAndUpdateOptions
}

export type MongoUpsertCommand<T extends MongoDocument> = {
    where?: Filter<T>
    document?: UpdateFilter<T>
    options?: FindOneAndUpdateOptions
}

export type FindOneAndUpsertCommand<T extends MongoDocument> = {
    where?: Filter<T>
    document?: UpdateFilter<T>
    options?: FindOneAndUpdateOptions
}

export type MongoInsertOneCommand<T extends MongoDocument> = {
    document: OptionalUnlessRequiredId<T>
    options?: InsertOneOptions
}

export type MongoInsertManyCommand<T extends MongoDocument> = {
    documents: Array<OptionalUnlessRequiredId<T>>
    options?: BulkWriteOptions
}

export type MongoCreateCommand<T extends MongoDocument> = {
    document: OptionalUnlessRequiredId<T>
}

export type MongoCreateIndexesCommand = {
    specification: Array<IndexDescription>
    options?: CreateIndexesOptions
}

export type MongoTransactionCommand = {
    wrapper: <T>(manager: TransactionManager) => Promise<T>
    options?: ClientSessionOptions
}

export type MatchStage<T extends MongoDocument> = {
    $match: Filter<T>
}

export type AggregationStage<T extends MongoDocument> = MatchStage<T>

export type MongoAggregateCommand<T extends MongoDocument> = {
    pipeline: Array<AggregationStage<T>>
    options?: AggregateOptions
}

export type MongoCountDocumentsCommand<T extends MongoDocument> = {
    where: Filter<T>
    options?: CountDocumentsOptions
}

export type MongoDropIndexCommand = {
    indexName: string
    options?: DropIndexesOptions
}
